@@ -1,4 +1,4 @@
-from .meta import MetaMessage, UnknownMetaMessage
+from .meta import MetaMessage, UnknownMetaMessage, KeySignatureError
 from .units import tick2second, second2tick, bpm2tempo, tempo2bpm
 from .tracks import MidiTrack, merge_tracks
 from .midifiles import MidiFile
