@@ -1,7 +1,7 @@
 """
 Meta messages for MIDI files.
 
-Todo:
+TODO:
      - what if an unknown meta message is implemented and someone depends on
        the 'data' attribute?
      - is 'type_byte' a good name?
@@ -20,62 +20,68 @@ from ..py2 import PY2
 
 _charset = 'latin1'
 
-def reverse_table(table):
+
+class KeySignatureError(Exception):
+    """ Raised when key cannot be converted from key/mode to key letter """
+    pass
+
+
+def _reverse_table(table):
     """Return value: key for dictionary."""
     return {value: key for (key, value) in table.items()}
 
-_key_signature_decode = {
-        (-7, 0): 'Cb',
-        (-6, 0): 'Gb',
-        (-5, 0): 'Db',
-        (-4, 0): 'Ab',
-        (-3, 0): 'Eb',
-        (-2, 0): 'Bb',
-        (-1, 0): 'F',
-        (0, 0): 'C',
-        (1, 0): 'G',
-        (2, 0): 'D',
-        (3, 0): 'A',
-        (4, 0): 'E',
-        (5, 0): 'B',
-        (6, 0): 'F#',
-        (7, 0): 'C#',
-        (-7, 1): 'Abm',
-        (-6, 1): 'Ebm',
-        (-5, 1): 'Bbm',
-        (-4, 1): 'Fm',
-        (-3, 1): 'Cm',
-        (-2, 1): 'Gm',
-        (-1, 1): 'Dm',
-        (0, 1): 'Am',
-        (1, 1): 'Em',
-        (2, 1): 'Bm',
-        (3, 1): 'F#m',
-        (4, 1): 'C#m',
-        (5, 1): 'G#m',
-        (6, 1): 'D#m',
-        (7, 1): 'A#m',
-    }
-_key_signature_encode = reverse_table(_key_signature_decode)
 
-_smpte_framerate_decode = {
-        0: 24,
-        1: 25,
-        2: 29.97,
-        3: 30,
-    }
-_smpte_framerate_encode = reverse_table(_smpte_framerate_decode)
+_key_signature_decode = {(-7, 0): 'Cb',
+                         (-6, 0): 'Gb',
+                         (-5, 0): 'Db',
+                         (-4, 0): 'Ab',
+                         (-3, 0): 'Eb',
+                         (-2, 0): 'Bb',
+                         (-1, 0): 'F',
+                         (0, 0): 'C',
+                         (1, 0): 'G',
+                         (2, 0): 'D',
+                         (3, 0): 'A',
+                         (4, 0): 'E',
+                         (5, 0): 'B',
+                         (6, 0): 'F#',
+                         (7, 0): 'C#',
+                         (-7, 1): 'Abm',
+                         (-6, 1): 'Ebm',
+                         (-5, 1): 'Bbm',
+                         (-4, 1): 'Fm',
+                         (-3, 1): 'Cm',
+                         (-2, 1): 'Gm',
+                         (-1, 1): 'Dm',
+                         (0, 1): 'Am',
+                         (1, 1): 'Em',
+                         (2, 1): 'Bm',
+                         (3, 1): 'F#m',
+                         (4, 1): 'C#m',
+                         (5, 1): 'G#m',
+                         (6, 1): 'D#m',
+                         (7, 1): 'A#m',
+                         }
+
+_key_signature_encode = _reverse_table(_key_signature_decode)
+
+_smpte_framerate_decode = {0: 24,
+                           1: 25,
+                           2: 29.97,
+                           3: 30,
+                           }
+
+_smpte_framerate_encode = _reverse_table(_smpte_framerate_decode)
 
 
 def signed(to_type, n):
-    formats = {
-        'byte': 'Bb',
-        'short': 'Hh',
-        'long': 'Ll',
-        'ubyte': 'bB',
-        'ushort': 'hH',
-        'ulong': 'lL',
-        }
+    formats = {'byte': 'Bb',
+               'short': 'Hh',
+               'long': 'Ll',
+               'ubyte': 'bB',
+               'ushort': 'hH',
+               'ulong': 'lL'
+               }
 
     try:
         pack_format, unpack_format = formats[to_type]
@@ -146,16 +152,18 @@ def check_int(value, low, high):
 
 
 if PY2:
-    def check_str(value):
-        if not isinstance(value, basestring):
-            raise TypeError('attribute must be unicode or string')
+    _STRING_TYPE = (str, unicode)  # noqa: F821
 else:
-    def check_str(value):
-        if not isinstance(value, str):
-            raise TypeError('attribute must a string')
+    _STRING_TYPE = str
+
+
+def check_str(value):
+    if not isinstance(value, _STRING_TYPE):
+        raise TypeError('attribute must be a string')
 
 
 class MetaSpec(object):
+    # The default is to do no checks.
     def check(self, name, value):
         pass
 
@@ -177,7 +185,7 @@ class MetaSpec_sequence_number(MetaSpec):
         return [message.number >> 8, message.number & 0xff]
 
     def check(self, name, value):
-        check_int(value, 0, 255)
+        check_int(value, 0, 0xffff)
 
 
 class MetaSpec_text(MetaSpec):
@@ -302,8 +310,8 @@ class MetaSpec_smpte_offset(MetaSpec):
                   'seconds',
                   'frames',
                   'sub_frames'
-                ]
-    # Todo: What are some good defaults?
+                  ]
+    # TODO: What are some good defaults?
     defaults = [24, 0, 0, 0, 0, 0]
 
     def decode(self, message, data):
@@ -339,12 +347,12 @@ class MetaSpec_smpte_offset(MetaSpec):
 
 class MetaSpec_time_signature(MetaSpec):
     type_byte = 0x58
-    # Todo: these need more sensible names.
+    # TODO: these need more sensible names.
     attributes = ['numerator',
                   'denominator',
                   'clocks_per_click',
                   'notated_32nd_notes_per_beat']
-    defaults = [4, 2, 24, 8]
+    defaults = [4, 4, 24, 8]
 
     def decode(self, message, data):
         message.numerator = data[0]
@@ -353,12 +361,11 @@ class MetaSpec_time_signature(MetaSpec):
         message.notated_32nd_notes_per_beat = data[3]
 
     def encode(self, message):
-        return [
-            message.numerator,
-            int(math.log(message.denominator, 2)),
-            message.clocks_per_click,
-            message.notated_32nd_notes_per_beat,
-            ]
+        return [message.numerator,
+                int(math.log(message.denominator, 2)),
+                message.clocks_per_click,
+                message.notated_32nd_notes_per_beat,
+                ]
 
     def check(self, name, value):
         if name == 'denominator':
@@ -382,14 +389,23 @@ class MetaSpec_key_signature(MetaSpec):
     def decode(self, message, data):
         key = signed('byte', data[0])
         mode = data[1]
-        message.key = _key_signature_decode[(key, mode)]
+        try:
+            message.key = _key_signature_decode[(key, mode)]
+        except KeyError:
+            if key < 7:
+                msg = ('Could not decode key with {} '
+                       'flats and mode {}'.format(abs(key), mode))
+            else:
+                msg = ('Could not decode key with {} '
+                       'sharps and mode {}'.format(key, mode))
+            raise KeySignatureError(msg)
 
     def encode(self, message):
         key, mode = _key_signature_encode[message.key]
         return [unsigned('byte', key), mode]
 
     def check(self, name, value):
-        if not value in _key_signature_encode:
+        if value not in _key_signature_encode:
             raise ValueError('invalid key {!r}'.format(value))
 
 
@@ -421,20 +437,22 @@ def add_meta_spec(klass):
 _META_SPECS = {}
 _META_SPEC_BY_TYPE = {}
 
+
 def _add_builtin_meta_specs():
     for name, spec in globals().items():
         if name.startswith('MetaSpec_'):
             add_meta_spec(spec)
 
+
 _add_builtin_meta_specs()
 
 
-def build_meta_message(type, data, delta=0):
-    # Todo: handle unknown type.
+def build_meta_message(meta_type, data, delta=0):
+    # TODO: handle unknown type.
     try:
-        spec = _META_SPECS[type]
+        spec = _META_SPECS[meta_type]
     except KeyError:
-        return UnknownMetaMessage(type, data)
+        return UnknownMetaMessage(meta_type, data)
     else:
         msg = MetaMessage(spec.type, time=delta)
 
@@ -448,7 +466,7 @@ class MetaMessage(BaseMessage):
     is_meta = True
 
     def __init__(self, type, **kwargs):
-        # Todo: handle unknown type?
+        # TODO: handle unknown type?
 
         spec = _META_SPEC_BY_TYPE[type]
         self_vars = vars(self)
@@ -513,9 +531,9 @@ class MetaMessage(BaseMessage):
         spec = _META_SPEC_BY_TYPE[self.type]
         data = spec.encode(self)
 
-        return ([0xff, spec.type_byte]
-                + encode_variable_int(len(data))
-                + data)
+        return ([0xff, spec.type_byte] +
+                encode_variable_int(len(data)) +
+                data)
 
     def __repr__(self):
         spec = _META_SPEC_BY_TYPE[self.type]
@@ -540,14 +558,16 @@ class UnknownMetaMessage(MetaMessage):
         vars(self).update({
             'type': 'unknown_meta',
             'type_byte': type_byte,
+            'data': data,
             'time': time})
 
     def __repr__(self):
         return ('<unknown meta message'
-                ' type_byte=0x{:02x} data={!r} time={}>').format(
-                self.type_byte,
-                self.data,
-                self.time)
+                ' type_byte=0x{:02x} '
+                'data={!r} time={}>').format(self.type_byte,
+                                             self.data,
+                                             self.time
+                                             )
 
     def __setattr__(self, name, value):
         # This doesn't do any checking.
@@ -555,6 +575,6 @@ class UnknownMetaMessage(MetaMessage):
         vars(self)[name] = value
 
     def bytes(self):
-        return ([0xff, self.type_byte]
-                + encode_variable_int(len(self.data))
-                + self.data)
+        return ([0xff, self.type_byte] +
+                encode_variable_int(len(self.data)) +
+                list(self.data))
